@@ -91,20 +91,23 @@ var generateFlags = []cli.Flag{
 	cli.StringSliceFlag{Name: "mounts-add", Usage: "configures additional mounts inside container"},
 	cli.StringSliceFlag{Name: "mounts-remove", Usage: "remove destination mountpoints from inside container"},
 	cli.BoolFlag{Name: "mounts-remove-all", Usage: "remove all mounts inside container"},
+	cli.StringFlag{Name: "oci-version", Usage: "specify the version of the Open Container Initiative runtime specification"},
 	cli.StringFlag{Name: "os", Value: runtime.GOOS, Usage: "operating system the container is created for"},
 	cli.StringFlag{Name: "output", Usage: "output file (defaults to stdout)"},
 	cli.BoolFlag{Name: "privileged", Usage: "enable privileged container settings"},
-	cli.StringSliceFlag{Name: "process-cap-add-ambient", Usage: "add Linux ambient capabilities"},
-	cli.StringSliceFlag{Name: "process-cap-add-bounding", Usage: "add Linux bounding capabilities"},
-	cli.StringSliceFlag{Name: "process-cap-add-effective", Usage: "add Linux effective capabilities"},
-	cli.StringSliceFlag{Name: "process-cap-add-inheritable", Usage: "add Linux inheritable capabilities"},
-	cli.StringSliceFlag{Name: "process-cap-add-permitted", Usage: "add Linux permitted capabilities"},
+	cli.StringFlag{Name: "process-cap-add", Usage: "add Linux capabilities to all 5 capability sets"},
+	cli.StringFlag{Name: "process-cap-add-ambient", Usage: "add Linux ambient capabilities"},
+	cli.StringFlag{Name: "process-cap-add-bounding", Usage: "add Linux bounding capabilities"},
+	cli.StringFlag{Name: "process-cap-add-effective", Usage: "add Linux effective capabilities"},
+	cli.StringFlag{Name: "process-cap-add-inheritable", Usage: "add Linux inheritable capabilities"},
+	cli.StringFlag{Name: "process-cap-add-permitted", Usage: "add Linux permitted capabilities"},
+	cli.StringFlag{Name: "process-cap-drop", Usage: "drop Linux capabilities to all 5 capability sets"},
 	cli.BoolFlag{Name: "process-cap-drop-all", Usage: "drop all Linux capabilities"},
-	cli.StringSliceFlag{Name: "process-cap-drop-ambient", Usage: "drop Linux ambient capabilities"},
-	cli.StringSliceFlag{Name: "process-cap-drop-bounding", Usage: "drop Linux bounding capabilities"},
-	cli.StringSliceFlag{Name: "process-cap-drop-effective", Usage: "drop Linux effective capabilities"},
-	cli.StringSliceFlag{Name: "process-cap-drop-inheritable", Usage: "drop Linux inheritable capabilities"},
-	cli.StringSliceFlag{Name: "process-cap-drop-permitted", Usage: "drop Linux permitted capabilities"},
+	cli.StringFlag{Name: "process-cap-drop-ambient", Usage: "drop Linux ambient capabilities"},
+	cli.StringFlag{Name: "process-cap-drop-bounding", Usage: "drop Linux bounding capabilities"},
+	cli.StringFlag{Name: "process-cap-drop-effective", Usage: "drop Linux effective capabilities"},
+	cli.StringFlag{Name: "process-cap-drop-inheritable", Usage: "drop Linux inheritable capabilities"},
+	cli.StringFlag{Name: "process-cap-drop-permitted", Usage: "drop Linux permitted capabilities"},
 	cli.StringFlag{Name: "process-consolesize", Usage: "specifies the console size in characters (width:height)"},
 	cli.StringFlag{Name: "process-cwd", Value: "/", Usage: "current working directory for the process"},
 	cli.IntFlag{Name: "process-gid", Usage: "gid for the process"},
@@ -126,10 +129,20 @@ var generateFlags = []cli.Flag{
 	cli.StringFlag{Name: "solaris-max-shm-memory", Usage: "Specifies the maximum amount of shared memory"},
 	cli.StringFlag{Name: "solaris-milestone", Usage: "Specifies the SMF FMRI"},
 	cli.StringFlag{Name: "template", Usage: "base template to use for creating the configuration"},
+	cli.StringSliceFlag{Name: "vm-hypervisor-parameters", Usage: "specifies an array of parameters to pass to the hypervisor"},
+	cli.StringFlag{Name: "vm-hypervisor-path", Usage: "specifies the path to the hypervisor binary that manages the container virtual machine"},
+	cli.StringFlag{Name: "vm-image-format", Usage: "set the format of the container virtual machine root image"},
+	cli.StringFlag{Name: "vm-image-path", Usage: "set path to the container virtual machine root image"},
+	cli.StringFlag{Name: "vm-kernel-initrd", Usage: "set path to an initial ramdisk to be used by the container virtual machine"},
+	cli.StringSliceFlag{Name: "vm-kernel-parameters", Usage: "specifies an array of parameters to pass to the kernel"},
+	cli.StringFlag{Name: "vm-kernel-path", Usage: "set path to the kernel used to boot the container virtual machine"},
+	cli.StringSliceFlag{Name: "windows-devices", Usage: "specifies a list of devices to be mapped into the container"},
 	cli.StringFlag{Name: "windows-hyperv-utilityVMPath", Usage: "specifies the path to the image used for the utility VM"},
 	cli.BoolFlag{Name: "windows-ignore-flushes-during-boot", Usage: "ignore flushes during boot"},
 	cli.StringSliceFlag{Name: "windows-layer-folders", Usage: "specifies a list of layer folders the container image relies on"},
 	cli.StringFlag{Name: "windows-network", Usage: "specifies network for container"},
+	cli.BoolFlag{Name: "windows-network-allowunqualifieddnsquery", Usage: "specifies networking unqualified DNS query is allowed"},
+	cli.StringFlag{Name: "windows-network-networkNamespace", Usage: "specifies network namespace for container"},
 	cli.StringFlag{Name: "windows-resources-cpu", Usage: "specifies CPU for container"},
 	cli.Uint64Flag{Name: "windows-resources-memory-limit", Usage: "specifies limit of memory"},
 	cli.StringFlag{Name: "windows-resources-storage", Usage: "specifies storage for container"},
@@ -192,6 +205,10 @@ func setupSpec(g *generate.Generator, context *cli.Context) error {
 		g.SetHostname(context.String("hostname"))
 	}
 
+	if context.IsSet("oci-version") {
+		g.SetOCIVersion(context.String("oci-version"))
+	}
+
 	if context.IsSet("label") {
 		annotations := context.StringSlice("label")
 		for _, s := range annotations {
@@ -330,91 +347,121 @@ func setupSpec(g *generate.Generator, context *cli.Context) error {
 		g.ClearProcessCapabilities()
 	}
 
+	if context.IsSet("process-cap-add") {
+		addCaps := context.String("process-cap-add")
+		parts := strings.Split(addCaps, ",")
+		for _, part := range parts {
+			if err := g.AddProcessCapability(part); err != nil {
+				return err
+			}
+		}
+	}
+
 	if context.IsSet("process-cap-add-ambient") {
-		addCaps := context.StringSlice("process-cap-add-ambient")
-		for _, cap := range addCaps {
-			if err := g.AddProcessCapabilityAmbient(cap); err != nil {
+		addCaps := context.String("process-cap-add-ambient")
+		parts := strings.Split(addCaps, ",")
+		for _, part := range parts {
+			if err := g.AddProcessCapabilityAmbient(part); err != nil {
 				return err
 			}
 		}
 	}
 
 	if context.IsSet("process-cap-add-bounding") {
-		addCaps := context.StringSlice("process-cap-add-bounding")
-		for _, cap := range addCaps {
-			if err := g.AddProcessCapabilityBounding(cap); err != nil {
+		addCaps := context.String("process-cap-add-bounding")
+		parts := strings.Split(addCaps, ",")
+		for _, part := range parts {
+			if err := g.AddProcessCapabilityBounding(part); err != nil {
 				return err
 			}
 		}
 	}
 
 	if context.IsSet("process-cap-add-effective") {
-		addCaps := context.StringSlice("process-cap-add-effective")
-		for _, cap := range addCaps {
-			if err := g.AddProcessCapabilityEffective(cap); err != nil {
+		addCaps := context.String("process-cap-add-effective")
+		parts := strings.Split(addCaps, ",")
+		for _, part := range parts {
+			if err := g.AddProcessCapabilityEffective(part); err != nil {
 				return err
 			}
 		}
 	}
 
 	if context.IsSet("process-cap-add-inheritable") {
-		addCaps := context.StringSlice("process-cap-add-inheritable")
-		for _, cap := range addCaps {
-			if err := g.AddProcessCapabilityInheritable(cap); err != nil {
+		addCaps := context.String("process-cap-add-inheritable")
+		parts := strings.Split(addCaps, ",")
+		for _, part := range parts {
+			if err := g.AddProcessCapabilityInheritable(part); err != nil {
 				return err
 			}
 		}
 	}
 
 	if context.IsSet("process-cap-add-permitted") {
-		addCaps := context.StringSlice("process-cap-add-permitted")
-		for _, cap := range addCaps {
-			if err := g.AddProcessCapabilityPermitted(cap); err != nil {
+		addCaps := context.String("process-cap-add-permitted")
+		parts := strings.Split(addCaps, ",")
+		for _, part := range parts {
+			if err := g.AddProcessCapabilityPermitted(part); err != nil {
+				return err
+			}
+		}
+	}
+
+	if context.IsSet("process-cap-drop") {
+		addCaps := context.String("process-cap-drop")
+		parts := strings.Split(addCaps, ",")
+		for _, part := range parts {
+			if err := g.DropProcessCapability(part); err != nil {
 				return err
 			}
 		}
 	}
 
 	if context.IsSet("process-cap-drop-ambient") {
-		dropCaps := context.StringSlice("process-cap-drop-ambient")
-		for _, cap := range dropCaps {
-			if err := g.DropProcessCapabilityAmbient(cap); err != nil {
+		dropCaps := context.String("process-cap-drop-ambient")
+		parts := strings.Split(dropCaps, ",")
+		for _, part := range parts {
+			if err := g.DropProcessCapabilityAmbient(part); err != nil {
 				return err
 			}
 		}
 	}
 
 	if context.IsSet("process-cap-drop-bounding") {
-		dropCaps := context.StringSlice("process-cap-drop-bounding")
-		for _, cap := range dropCaps {
-			if err := g.DropProcessCapabilityBounding(cap); err != nil {
+		dropCaps := context.String("process-cap-drop-bounding")
+		parts := strings.Split(dropCaps, ",")
+		for _, part := range parts {
+			if err := g.DropProcessCapabilityBounding(part); err != nil {
 				return err
 			}
 		}
 	}
 
 	if context.IsSet("process-cap-drop-effective") {
-		dropCaps := context.StringSlice("process-cap-drop-effective")
-		for _, cap := range dropCaps {
-			if err := g.DropProcessCapabilityEffective(cap); err != nil {
+		dropCaps := context.String("process-cap-drop-effective")
+		parts := strings.Split(dropCaps, ",")
+		for _, part := range parts {
+			if err := g.DropProcessCapabilityEffective(part); err != nil {
 				return err
 			}
 		}
 	}
 
 	if context.IsSet("process-cap-drop-inheritable") {
-		dropCaps := context.StringSlice("process-cap-drop-inheritable")
-		for _, cap := range dropCaps {
-			if err := g.DropProcessCapabilityInheritable(cap); err != nil {
+		dropCaps := context.String("process-cap-drop-inheritable")
+		parts := strings.Split(dropCaps, ",")
+		for _, part := range parts {
+			if err := g.DropProcessCapabilityInheritable(part); err != nil {
 				return err
 			}
 		}
 	}
 
 	if context.IsSet("process-cap-drop-permitted") {
-		dropCaps := context.StringSlice("process-cap-drop-permitted")
-		for _, cap := range dropCaps {
-			if err := g.DropProcessCapabilityPermitted(cap); err != nil {
+		dropCaps := context.String("process-cap-drop-permitted")
+		parts := strings.Split(dropCaps, ",")
+		for _, part := range parts {
+			if err := g.DropProcessCapabilityPermitted(part); err != nil {
 				return err
 			}
 		}
@@ -851,12 +898,50 @@ func setupSpec(g *generate.Generator, context *cli.Context) error {
 		g.SetSolarisMilestone(context.String("solaris-milestone"))
 	}
 
+	if context.IsSet("vm-hypervisor-path") {
+		if err := g.SetVMHypervisorPath(context.String("vm-hypervisor-path")); err != nil {
+			return err
+		}
+	}
+
+	if context.IsSet("vm-hypervisor-parameters") {
+		g.SetVMHypervisorParameters(context.StringSlice("vm-hypervisor-parameters"))
+	}
+
+	if context.IsSet("vm-kernel-path") {
+		if err := g.SetVMKernelPath(context.String("vm-kernel-path")); err != nil {
+			return err
+		}
+	}
+
+	if context.IsSet("vm-kernel-parameters") {
+		g.SetVMKernelParameters(context.StringSlice("vm-kernel-parameters"))
+	}
+
+	if context.IsSet("vm-kernel-initrd") {
+		if err := g.SetVMKernelInitRD(context.String("vm-kernel-initrd")); err != nil {
+			return err
+		}
+	}
+
+	if context.IsSet("vm-image-path") {
+		if err := g.SetVMImagePath(context.String("vm-image-path")); err != nil {
+			return err
+		}
+	}
+
+	if context.IsSet("vm-image-format") {
+		if err := g.SetVMImageFormat(context.String("vm-image-format")); err != nil {
+			return err
+		}
+	}
+
 	if context.IsSet("windows-hyperv-utilityVMPath") {
 		g.SetWindowsHypervUntilityVMPath(context.String("windows-hyperv-utilityVMPath"))
 	}
 
 	if context.IsSet("windows-ignore-flushes-during-boot") {
-		g.SetWinodwsIgnoreFlushesDuringBoot(context.Bool("windows-ignore-flushes-during-boot"))
+		g.SetWindowsIgnoreFlushesDuringBoot(context.Bool("windows-ignore-flushes-during-boot"))
 	}
 
 	if context.IsSet("windows-layer-folders") {
@@ -866,6 +951,19 @@ func setupSpec(g *generate.Generator, context *cli.Context) error {
 		}
 	}
 
+	if context.IsSet("windows-devices") {
+		devices := context.StringSlice("windows-devices")
+		for _, device := range devices {
+			id, idType, err := parseWindowsDevices(device)
+			if err != nil {
+				return err
+			}
+			if err := g.AddWindowsDevices(id, idType); err != nil {
+				return err
+			}
+		}
+	}
+
 	if context.IsSet("windows-network") {
 		network := context.String("windows-network")
 		tmpNetwork := rspec.WindowsNetwork{}
@@ -875,6 +973,14 @@ func setupSpec(g *generate.Generator, context *cli.Context) error {
 		g.SetWindowsNetwork(tmpNetwork)
 	}
 
+	if context.IsSet("windows-network-allowunqualifieddnsquery") {
+		g.SetWindowsNetworkAllowUnqualifiedDNSQuery(context.Bool("windows-network-allowunqualifieddnsquery"))
+	}
+
+	if context.IsSet("windows-network-networkNamespace") {
+		g.SetWindowsNetworkNamespace(context.String("windows-network-networkNamespace"))
+	}
+
 	if context.IsSet("windows-resources-cpu") {
 		cpu := context.String("windows-resources-cpu")
 		tmpCPU := rspec.WindowsCPUResources{}
@@ -899,7 +1005,7 @@ func setupSpec(g *generate.Generator, context *cli.Context) error {
 	}
 
 	if context.IsSet("windows-servicing") {
-		g.SetWinodwsServicing(context.Bool("windows-servicing"))
+		g.SetWindowsServicing(context.Bool("windows-servicing"))
 	}
 
 	err := addSeccomp(context, g)
@@ -1013,6 +1119,17 @@ func parseNamespace(ns string) (string, string, error) {
 	return nsType, nsPath, nil
 }
 
+func parseWindowsDevices(device string) (string, string, error) {
+	parts := strings.Split(device, ":")
+	if len(parts) != 2 {
+		return "", "", fmt.Errorf("invalid windows device value: %s", device)
+	}
+
+	id := parts[0]
+	idType := parts[1]
+	return id, idType, nil
+}
+
 var deviceType = map[string]bool{
 	"b": true, // a block (buffered) special file
 	"c": true, // a character special file
