@@ -43,7 +43,7 @@ func TestJSONSchema(t *testing.T) {
 	}{
 		{
 			config: &rspec.Spec{},
-			error:  "1 error occurred:\n\n* Version string empty\nRefer to: https://github.com/opencontainers/runtime-spec/blob/v1.0.1/config.md#specification-version",
+			error:  "1 error occurred:\n\t* Version string empty\nRefer to: https://github.com/opencontainers/runtime-spec/blob/v1.0.1/config.md#specification-version\n\n",
 		},
 		{
 			config: &rspec.Spec{
@@ -128,7 +128,7 @@ func TestJSONSchema(t *testing.T) {
 					},
 				},
 			},
-			error: "2 errors occurred:\n\n* linux.namespaces.0: Must validate at least one schema (anyOf)\n* linux.namespaces.0.type: linux.namespaces.0.type must be one of the following: \"mount\", \"pid\", \"network\", \"uts\", \"ipc\", \"user\", \"cgroup\"",
+			error: "2 errors occurred:\n\t* linux.namespaces.0: Must validate at least one schema (anyOf)\n\t* linux.namespaces.0.type: linux.namespaces.0.type must be one of the following: \"mount\", \"pid\", \"network\", \"uts\", \"ipc\", \"user\", \"cgroup\"\n\n",
 		},
 		{
 			config: &rspec.Spec{
@@ -510,7 +510,7 @@ func TestCheckProcess(t *testing.T) {
 
 func TestCheckLinux(t *testing.T) {
 	weightDevices := []rspec.LinuxWeightDevice{
-		rspec.LinuxWeightDevice{},
+		{},
 	}
 	weightDevices[0].Major = 5
 	weightDevices[0].Minor = 0
@@ -716,11 +716,11 @@ func TestCheckMandatoryFields(t *testing.T) {
 	}{
 		{
 			config: &rspec.Spec{},
-			error:  "1 error occurred:\n\n* 'Spec.Version' should not be empty",
+			error:  "1 error occurred:\n\t* 'Spec.Version' should not be empty\n\n",
 		},
 		{
 			config: nil,
-			error:  "1 error occurred:\n\n* Spec can't be nil",
+			error:  "1 error occurred:\n\t* Spec can't be nil\n\n",
 		},
 		{
 			config: &rspec.Spec{
@@ -733,7 +733,7 @@ func TestCheckMandatoryFields(t *testing.T) {
 				Version: "1.0.0",
 				Root:    &rspec.Root{},
 			},
-			error: "1 error occurred:\n\n* 'Root.Path' should not be empty",
+			error: "1 error occurred:\n\t* 'Root.Path' should not be empty\n\n",
 		},
 	} {
 		t.Run(tt.error, func(t *testing.T) {
